from time import *
from math import ceil,floor
from datetime import datetime


############################################################################# column properties
class genericColumn(object):
   """
   This class collection represents all the possible columns: including the caption of the columns and the decorators to be applied to the data
   """
   colName='colName'        # column title
   attrNames=["attrNames",] # name of the key in DAS dictionary output
   def __init__(self,path='',attrNames=[]):
     if len(path)>0: self.path=path
     if len(attrNames)>0: self.attrNames=attrNames
     
   def decorator(self,values): 
     """
     Operations to be performed on the values in order to print them
     """ 
     if len(self.attrNames)==1:
       return str(values[self.attrNames[0]])
     else:
       for attr in self.attrNames:
         try:
           return str(values[attr])
         except:
           pass
     return "Error: attribute not found"
          
    
   def humanFilesize(self,size):
    """
    Convert size in Bytes to more human readable format
    """
    if size < 1024:
        return ("%d B") % (size)
    elif size < (1024 * 1024):
        return ("%.2f KB") % (float(size) / (1024))
    elif size < (1024 * 1024 * 1024):
        return ("%.2f MB") % (float(size) / (1024 * 1024))
    else:
        return ("%.2f GB") % (float(size) / (1024 * 1024 * 1024))

########################################## 
class RunId(genericColumn):
   path=""
   colName='Run ID'
   attrNames=["run_id"]
   def decorator(self,values):
    return """<a href="run?run_id=%s">%s</a>"""%(values[self.attrNames[0]],values[self.attrNames[0]])
   
class JobId(genericColumn):
   colName='Job ID'
   attrNames=["job_id"]
   def decorator(self,values):
    return "<a href=\""+self.path+"Jobs&Id=%s\">%s</a>"%(values[self.attrNames[0]],values[self.attrNames[0]])
   
class FileId(JobId):
   colName='Id'
   attrNames=["id"]
   def decorator(self,values):
    return "<a href=\""+self.path+"Files&Id=%s\">%s</a>"%(values[self.attrNames[0]],values[self.attrNames[0]])
   
class StreamerId(JobId):
   colName='Id'
   attrNames=["streamer_id"]
   def decorator(self,values):
    return "<a href=\""+self.path+"Files&Id=%s\">%s</a>"%(values[self.attrNames[0]],values[self.attrNames[0]])
        #,filesize(),events(),dataset(),exportable(),lfn()

class lumiId(genericColumn):
   colName='Lumi Id'
   attrNames=["lumi_id"]
   
class nstreamers(genericColumn):
   colName='Streamers'
   attrNames=["num_streamers"]
   
class stream(genericColumn):
   colName='Stream'
   attrNames=["stream"]
      
class tier(genericColumn):
   colName='Tier'
   attrNames=["tier"]      
   
class skim(genericColumn):
   colName='Skim'
   attrNames=["skim"]
         
   
class dataset(genericColumn):
   colName='Dataset'
   attrNames=["dataset"]
         
class sTime(genericColumn):
   colName='Start'
   attrNames=["start_time"]
   def decorator(self,values):
     str= """ &nbsp;<a class="info" href="#"><img src="imgs/clock.png" alt="" style="border: 0px none ;"><span style="width: 130px; left: 0px;">%s</span></a>"""
     return str%(strftime("%Y-%m-%d %H:%M:%S %Z", localtime(int(values[self.attrNames[0]]))))  
     #return str%(ctime(int(values[0])))
   
class eTime(genericColumn):
   colName='End'
   attrNames=["end_time"]
   def decorator(self,values):
    str= """ &nbsp;<a class="info" href="#"><img src="imgs/clock.png" alt="" style="border: 0px none ;"><span style="width: 130px; left: 0px;">%s</span></a>"""
    return str%(strftime("%Y-%m-%d %H:%M:%S %Z", localtime(int(values[self.attrNames[0]]))))
    #return str%(ctime(int(values[0])))
   
class defTime(genericColumn):
   colName='Definition Time'
   attrNames=["def_time"]
   def decorator(self,values):
      return strftime("%Y-%m-%d %H:%M:%S %Z", localtime(int(values[self.attrNames[0]])))

class runTime(genericColumn):
   colName='Running Time'
   attrNames=["state_time"]
   def decorator(self,values):
    kk=mktime(gmtime())-int(values[self.attrNames[0]])
    return strftime("%Hh %Mm %Ss", localtime(kk))
   
class filesNum(genericColumn):
   colName='# of files'
   attrNames=["inputfiles"]
   
class filesize(genericColumn):
   colName='Filesize'
   attrNames=["filesize"]
   def decorator(self,values):
    return self.humanFilesize(int(values[self.attrNames[0]]))  
 
class exportable(genericColumn):
   colName='Exportable'
   attrNames=["streamer_exp"]
         
class process(genericColumn):
   colName='Process'
   attrNames=["process"]
  
class version(genericColumn):
   colName='Online Base'
   attrNames=["version"]
   def decorator(self,values): 
     return str(values[self.attrNames[0]].replace("CMSSW_","")) 
    
class expVersion(genericColumn):
   colName='Express Version'
   attrNames=["expversion"]
   def decorator(self,values): 
     return str(values[self.attrNames[0]].replace("CMSSW_","")) 
    
   
class repVersion(genericColumn):
   colName='Repack Version'
   attrNames=["repversion"]
   def decorator(self,values): 
     return str(values[self.attrNames[0]].replace("CMSSW_","")) 
    

class hlt(genericColumn):
   colName='HLTKey'
   attrNames=["hltkey"]
   def decorator(self,values):
    l=len(values[self.attrNames[0]])
    if l>10: string="..."+values[self.attrNames[0]][l-10:l]
    else: string=values[self.attrNames[0]]
    return  """<a href="http://cern.ch/confdb/browser/convert2Html.jsp?dbName=ORCOFF&configName=%s" title="%s"> %s</a>"""%(values[self.attrNames[0]],values[self.attrNames[0]],string)
    
   
class acq(genericColumn):
   colName='ACQ Era'
   attrNames=["acq_era"]

class status(genericColumn):
   colName='Status'
   attrNames=["status"]
   
class events(genericColumn):
   colName='Events'
   attrNames=["events"] 
     
class lfn(genericColumn):
   colName='Lfn'
   attrNames=["lfn"]
   def decorator(self,values):
    return  "<a class=\"info\" href=\"file:%s\"><img src=\"imgs/doc.png\"  alt=\"\" border=\"0\" height=\"12\"><span>%s</span></a>"""%(values[self.attrNames[0]],values[self.attrNames[0]])

class splitInProgress(genericColumn):
   colName="Split in Progress"
   attrNames=["splitinprocessing"]
   
class procVersion(genericColumn):
   colName="Proc Version"
   attrNames=["proc_version"]
   
class globalTag(genericColumn):
   colName="Global Tag"
   attrNames=["global_tag"]
   
class scenario(genericColumn):
   colName='Scenario'
   attrNames=["scenario","ro_scenario"]
   
class confUrl(genericColumn):
   colName="Config  Url"
   attrNames=["config_url"]
   def decorator(self,values):
    #print values[self.attrNames[0]]
    if values[self.attrNames[0]]==None: return ""
    l=len(values[self.attrNames[0]])
    if l>40: string="..."+values[self.attrNames[0]][l-40:l]
    else: string=values[self.attrNames[0]]
    return  """<a href="%s">%s</a>"""%(values[self.attrNames[0]],string)
   
class plotLinks(genericColumn):
   colName="Plots"
   attrNames=["run_id"]
   def decorator(self,values):
    #dt=datetime.fromtimestamp(min(mktime(localtime()),values[self.attrNames[1]]+47*60*60))
    #params="&sy=%d&sm=%d&sd=%d&sh=%d"%(dt.year,dt.month,dt.day,dt.hour)
    return  """<a href="statplots?run_id=%s&autotime=1"> <img border="0" src="imgs/plot.png" alt="run-plots" title="statistical plots for run: %s" width="20"></a>""" %(values[self.attrNames[0]],values[self.attrNames[0]])
   
  
####################################################

#values = list with attrNames + auxInfo
class graphColumn(genericColumn):
  option="?run=%s"
  MAXLEN=0.8
 
  def decorator(self,values):
    #percentuals=str(values)
    percentuals=self.getPercentuals(values)
    return percentuals
      
  def getPercentuals(self,values):
    percentuals=""
    try:
     percentuals+=self.printRGPercentual(values[self.attrNames[0]],values[self.attrNames[1]])
     percentuals+=self.printRGPercentual(values[self.attrNames[2]],values[self.attrNames[3]])
     percentuals+=self.printPercentual(values[self.attrNames[4]],values[self.attrNames[5]])
    except Exception,e :
      #print e
      pass
    return percentuals
  
  def printRGPercentual(self,num,denom):
    try:
     completed=num['completed_count']
     failed=num['failed_count']
     denomN=denom['count']
     numN=completed+failed
     #### perc= percentual of completed computation
     if denomN>0:
      perc= ceil(100*numN//denomN)
     else: perc=0
     #print "perc=%d"%perc
     if numN<1:numN=1
     #######string
     if perc==100:    
       percStr="1"
     else:
          if perc==0 and denomN==0:
             percStr="None"
          else: percStr="%d%%"%(perc)
     cPerc=ceil(self.MAXLEN*perc*completed//numN) ## Green bar length
     fPerc=ceil(self.MAXLEN*perc*failed//numN)    ## Red bar length
     ### show the smallest possible bar if failed>0
     if failed>0:
       fPerc=max(fPerc,1)
     res="""<a href="#" title="(%d+%d) /%d"><img src="imgs/barG.png"  alt="" height="12" width="%d" border="0"> 
           <img src="imgs/barR.png"  alt="" height="12" width="%d" border="0"> </a> %s """%( completed,failed,denomN,
           cPerc,fPerc,percStr)
       ##print percentuals
     res+="<br/>"
    except Exception, e:
      #print e
      # may be the value is count=0 by default
      res=self.printPercentual(num,denom)
    return res
    
  def printPercentual(self,num,denom):
    try:
     denomN=denom['count']
     numN=num['count']
     if denomN>0:
      perc= ceil(100*numN//denomN)
     else: perc=0
     if perc==100:    percStr="1"
     else:
          if perc==0 and denomN==0: percStr="None"
          else: percStr="%d%%"%(perc)
     res="""<a href="#" title="%d /%d"><img src="imgs/barG.png"  alt="" height="12" width="%d" border="0"></a> &nbsp;%s """%(
           numN,denomN,ceil(self.MAXLEN*perc), percStr)
       ##print percentuals
     res+="<br/>"
    except Exception, e:
      #print str(e)
      res=str(num)+"---"+str(denom)+"<BR>"
    return res
  
################################################################################# 
# the name of the variables in 'attrNames' and 'auxSuffixes' are the name of the services request/file in DAS to be called
# They are called in the following order:
# 1) attrNames  [these attributes are already present in the the general data: usually they are fetched globally at the begininning of the run detail]
# 2) auxSuffixes [these attributes require a different DAS call for every run]
   
# attrNames are the same information fetched/processed  in runDetails
class numstreamers(graphColumn):
   colName='Streamers'
   attrNames=['numstreamers']
   auxSuffixes=[]
   def decorator(self,values):
       return str(values[self.attrNames[0]]['count'])

class expressGraph(graphColumn):
   colName='Express Graph'
   attrNames=['numexprcompleted','numexpresstotal','numexpresstotalstreamers'] #'numexpressmerged','numexpressmergetotal'
   #'numexprstreamers',
   auxSuffixes=['numexpressstreamersbyrun']
   def getPercentuals(self,values):
    percentuals=""
    try:
     percentuals+=self.printRGPercentual(values[self.auxSuffixes[0]],values[self.attrNames[2]])
     percentuals+=self.printRGPercentual(values[self.attrNames[0]],values[self.attrNames[1]])
    except Exception, e:
      #print str(e)
      pass
    return percentuals

class repackGraph(graphColumn):
   colName='Repacked Graph'
   attrNames=['numstreamers','numexpresstotalstreamers','numrepackcompleted','numrepackedtotal','numrepackmergedexported','numrepackmerged','numrepacktemp']
   #auxSuffixes=['numrepacktempbyrun']
   #'numrepacktemp',
   def getPercentuals(self,values):
    percentuals=""
    try:
     percentuals+=self.printRGPercentual(values[self.attrNames[6]],self.getDiff(values[self.attrNames[0]],values[self.attrNames[1]])) # values[self.attrNames[0]] -
     percentuals+=self.printRGPercentual(values[self.attrNames[2]],values[self.attrNames[3]])
     percentuals+=self.printPercentual(values[self.attrNames[4]],values[self.attrNames[5]])
    except Exception, e:
      #print str(e)
      pass
    return percentuals
   
   def getDiff(self,dict1,dict2):    
       return {'count':int(dict1['count'])-int(dict2['count'])}
   
class recoGraph(graphColumn):
   colName='Reconstructed Graph'
   attrNames=['numrecotemp','numrecotemptotal','numrecocompleted','numrecototal','numrecomergedexported','numrecomerged']
   auxSuffixes=[]
   
     
class alcaGraph(recoGraph):
   colName='Alcaskimmed Graph'
   auxSuffixes=[]
   attrNames=['numalcatemp','numalcatemptotal','numalcacompleted','numalcatotal','numalcamergedexported','numalcamerged']
    
    
####################### special columns for configuration tables:
   
class rpProcVersion(genericColumn):
   colName="REPACK:Proc Version"
   attrNames=["rp_proc_version"]
   
class roProcVersion(genericColumn):
   colName="RECO:Proc Version"
   attrNames=["ro_proc_version"]
   
class alProcVersion(genericColumn):
   colName="ALCA:Proc Version"
   attrNames=["al_proc_version"]

class roReco(genericColumn):
   colName="Reco"
   attrNames=["ro_do_reco"]
   
class alAlcaskim(genericColumn):
   colName="Alcaskim"
   attrNames=["al_do_alca"]
   
   
class roCmsswVersion(genericColumn):
   colName="RECO: Cmssw Version"
   attrNames=["ro_cmssw_version"]
   
class alCmsswVersion(genericColumn):
   colName="ALCA: Cmssw Version"
   attrNames=["al_cmssw_version"]   
   
class roGlobalTag(genericColumn):
   colName="RECO:Global Tag"
   attrNames=["ro_global_tag"]
   
class roConfUrl(confUrl):
   colName="RECO:Config  Url"
   attrNames=["ro_config_url"]
         
class alConfUrl(confUrl):
   colName="ALCA:Config  Url"
   attrNames=["al_config_url"]  
   
class AMConfUrl(confUrl):
   colName="Alca Merge Config  Url"
   attrNames=["alcamerge_config_url"]

