"""
_MotT0Index_
Generic class representing the index page of the MotT0 monitoring system.
 
""" 
from WMCore.WebTools.Page import TemplatedPage
 

from Table import *
from TableContents import *

from Statistics import *
from Heartbeats import * 
from RunDetails import *
from HTTPfetcher import *

import threading, os,time 
from Utils import *
from MotT0 import MotT0Page

class MotT0CachedIndex(MotT0Page):
    
    def getTemplate(self):
       #security check on refreshing flag (constants from Utils)
       if fileOlderThen(REFRESH_FLAG_FILE,REFRESH_TIMEOUT):
           print "Removing refreshing flag"
           removeFile(REFRESH_FLAG_FILE)
           
       eParams={
            "title": "Error",
            "time" : ctime(),
            "tmplPath": self.templatedir,
            "msg": "No cached page found, a new one is being created right now. Refresh the page in few minutes, please "
              } 
       if existsFile(CACHE_FILE):
         #cacheTime=str(self.getLastTimeCached())
         content=getCachedFile(CACHE_FILE)
         if not(workingDAS(self.DASUrl)):
           content=content.replace("&nbsp;&nbsp;&nbsp;</p>","&nbsp;&nbsp;&nbsp;</p> \
                      <div align=\"center\"><B> Error testing DAS Service<BR> Support has been informed. </B></div>")
           sendAlarm(self.motT0Template.config.emailAlert,self.DASUrl,"Impossible to connect to DAS")
           
         if fileOlderThen(CACHE_FILE,60*60):
          content=content.replace("<p>T0 status @","<p><B><table  class=\"clear\" ><tr><td style=\"background-color: white;\">T0 status @</td><td style=\"background-color: red;\">",1)
          content=content.replace("&nbsp;&nbsp;&nbsp;","</td></tr></table> Caution the page might be old.&nbsp;&nbsp;&nbsp;",1)
         
         cParams={ 
            "content":content
            }
         if not(existsFile(REFRESH_FLAG_FILE)):
           setFlag(REFRESH_FLAG_FILE)
           t=MotT0IndexThread(self.motT0Template)
           t.start()
         return self.motT0Template.templatepage('cached', **cParams )
           
       else:
         if not(existsFile(REFRESH_FLAG_FILE)): 
           setFlag(REFRESH_FLAG_FILE) # this flag will be disabled by the thread that accomplishes the result
           t=MotT0IndexThread(self.motT0Template)
           t.start()
         return self.motT0Template.templatepage('error', **eParams )
             
#################################################################################################################
class MotT0IndexThread(threading.Thread):
 
    def __init__ ( self, templatePage,saveData=True):
      self.saveData=saveData
      self.table=MotT0Index(templatePage)
      threading.Thread.__init__ ( self )
    
    def run(self):
       self.data= self.table.getTemplate()
       if self.data!=None and len(self.data)>10000:
         print "len="+str(len(self.data))
         if self.saveData:
           saveDataInFile(self.data,CACHE_FILE)
       removeFile(REFRESH_FLAG_FILE)
           
##########################################################################################
class MotT0Index(MotT0Page):
    page=""
    heartbeatsCode=""
    runsDetailsCode=""
    statsCode=""
    exprJ=""
    repkJ=""
    recoJ=""
    alcaJ=""
    exprMJ=""
    repkMJ=""
    recoMJ=""
    alcaMJ=""    
    title="T0Mon main page"
    
    def getTemplate(self):
      #see TableContents for table content information
      contents=[exprJ(),exprMJ(),repkJ(),repkMJ(),recoJ(),recoMJ(),alcaJ(),alcaMJ()]
      #generate code for tables
      threads=[]
      try:
        h = HeartThread(self.DASUrl)
        threads.append(h) 
        h.start()
        
        content=runDetails({})
        r = tableThread(self.DASUrl,content)
        threads.append(r)
        r.start()
        
        s= StatThread(self.DASUrl)
        threads.append(s)
        s.start()
        ##print " Jobs tables"
        for n in range(0,len(contents),1):
            #print "creating "+ str(contents[c].name)+"  "+ str(contents[c+1].name)
            c=contents[n]
            tt=tableThread(self.DASUrl,c)
            threads.append(tt)
            tt.start() 
        # wait for the threads to end  
        for t in threads:
            t.join()
        # collect and store results
        for c in range(len(contents)-1,-1,-1):
            ##print "Reading "+str(contents[c].name)
            data=threads.pop().getHTML()
            if data=="": raise Exception("Data missing")
            setattr(self, str(contents[c].name)+'',data)
        self.statsCode= threads.pop().getHTML()
        self.runsDetailsCode= threads.pop().getHTML()
        self.heartbeatsCode =  threads.pop().getHTML()
      except Exception, e:
         print e
         ##
      params={
            "page": "system?",
            "title": self.title,
            "time" : ctime(),
            "tmplPath": self.templatedir,
            "heartbeats": self.heartbeatsCode,
            "runsT":self.runsDetailsCode,
            'stats':self.statsCode,
            'exprJ':self.exprJ,
            'repkJ':self.repkJ,
            'recoJ':self.recoJ,
            'alcaJ':self.alcaJ,
            'exprMJ':self.exprMJ,
            'repkMJ':self.repkMJ,
            'recoMJ':self.recoMJ,
            'alcaMJ':self.alcaMJ
              }
      return self.motT0Template.templatepage('system', **params )
